/**
 * 
 */
package org.springblade.cube.controller;

import com.alibaba.excel.EasyExcel;
import com.alibaba.excel.support.ExcelTypeEnum;
import com.alibaba.fastjson2.JSON;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.io.FileUtils;
import org.apache.commons.io.FilenameUtils;
import org.springblade.core.boot.ctrl.BladeController;
import org.springblade.core.secure.utils.SecureUtil;
import org.springblade.core.tool.api.R;
import org.springblade.cube.dto.ExcelFormParam;
import org.springblade.cube.exception.ExcelReadLimitException;
import org.springblade.cube.model.CubeTableFile;
import org.springblade.cube.model.FileStructure;
import org.springblade.cube.service.CubeTableFileService;
import org.springblade.cube.service.DynamicTableService;
import org.springblade.cube.utils.CubeConstants;
import org.springblade.cube.utils.CubeExcelAnalysisEventListener;
import org.springblade.cube.utils.CubeUtils;
import org.springblade.datasource.constants.DbConstant;
import org.springblade.datasource.context.DialectContext;
import org.springblade.datasource.enums.DetailColumnTypeEnum;
import org.springblade.datasource.enums.DsTypeEnum;
import org.springblade.datasource.enums.ViewColumnTypeEnum;
import org.springblade.datasource.model.TableColumnDto;
import org.springblade.report.entity.ReportDatasource;
import org.springblade.report.service.IReportDatasourceService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.multipart.MultipartFile;

import java.io.File;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Map;
import java.util.UUID;
import java.util.stream.Collectors;
import java.util.stream.Stream;

/**
 * @author wookvn
 *
 */
@Slf4j
@RestController
@RequestMapping("/cube/file")
@Api(value = "数据集文件接口", tags = "数据集_文件上传接口")
public class CubeFileController extends BladeController {

	@Autowired
	private IReportDatasourceService reportDatasourceService;

	@Autowired
	private DynamicTableService dynamicTableService;

	@Autowired
	private CubeTableFileService cubeTableFileService;
	
	@Autowired
	private DialectContext dialectContext;
	
	@Value("${blade.file.uploadCube}")
	private String cubeDir;

	/**
	 * 文件结构
	 * 
	 * @param file
	 * @return
	 * @throws Exception
	 */
	@ApiOperation(value = "文件结构接口")
	@PostMapping("/structure")
	public R<List<FileStructure>> fileStructure(MultipartFile file) throws Exception {
		// 1.读取并解析excel文件
		String extension = FilenameUtils.getExtension(file.getOriginalFilename());
		if (!Stream.of(CubeConstants.SUPPORTS_FILE).anyMatch(f -> f.equalsIgnoreCase(extension))) {
			return R.fail(String.format("文件只支持%s格式", String.join(",", CubeConstants.SUPPORTS_FILE)));
		}
		// 数据记录集
		List<Map<Integer, String>> dataList = new ArrayList<Map<Integer, String>>();
		CubeExcelAnalysisEventListener analysisEventListener = new CubeExcelAnalysisEventListener(CubeConstants.DEFAULT_READ_EXCEL_SIZE, false, dataList::addAll);
		try {
			// 读取数据
			EasyExcel.read(file.getInputStream(), analysisEventListener).excelType(ExcelTypeEnum.valueOf(extension.toUpperCase())).sheet().doRead();
		} catch (Exception e) {
			if (e.getCause() instanceof ExcelReadLimitException || e instanceof ExcelReadLimitException) {
			} else {
				log.error(e.getMessage(), e);
			}
		}
		// 表头数据
		Map<Integer, String> headDataMap = analysisEventListener.getHeadMap();
		List<FileStructure> fileStructList = new ArrayList<FileStructure>();
		// 表头和数据集不为空
		if (headDataMap != null && !headDataMap.isEmpty()) {
			headDataMap.forEach((k, v) -> {
				FileStructure fileStructure = new FileStructure();
				fileStructure.setPinyinAllColumnName(CubeUtils.getPinyin(v, "_"));
				fileStructure.setPinyinFirstLetterColumnName(CubeUtils.getPinyinFirstLetter(v));
				fileStructure.setSystemColumnName(CubeConstants.COLUMN_NAME_PREFIX + k);
				fileStructure.setColumnName(fileStructure.getPinyinAllColumnName());
				fileStructure.setColumnType(dataList.isEmpty() ? ViewColumnTypeEnum.STRING : CubeUtils.resolveDataType(k, dataList));
				fileStructure.setComment(v);
				fileStructure.setColumnIndex(k);
				if(fileStructure.getColumnType() == ViewColumnTypeEnum.STRING) {
					fileStructure.setColumnSize(DbConstant.DEFAULT_COLUMN_STRING_SIZE); //字符串默认长度
				}else if(fileStructure.getColumnType() == ViewColumnTypeEnum.NUMBER) { //数字类型的全部：默认为浮点数字
					fileStructure.setColumnSize(DbConstant.DEFAULT_COLUMN_NUMBER_SIZE); //数字精度
					fileStructure.setDecimalDigits(DbConstant.DEFAULT_COLUMN_NUMBER_DIGITS); //小数点位数
				}
				fileStructList.add(fileStructure);
			});
		}
		R<List<FileStructure>> data = R.data(fileStructList);
		data.setExt("ltb_"+CubeUtils.getPinyin(file.getOriginalFilename(), "_"));
		return data;
	}

	/**
	 * 文件上传创建表
	 * 
	 * @return
	 */
	@ApiOperation(value = "文件上传创建表")
	@PostMapping("/uploadToDs")
	@ApiImplicitParams({ 
			@ApiImplicitParam(name = "file", value = "文件", required = true),
			@ApiImplicitParam(name = "tableName", value = "表名", dataType = "string", required = true),
			@ApiImplicitParam(name = "tableComment", value = "表注释", required = true),
			@ApiImplicitParam(name = "dsId", value = "数据源ID", required = true),
			@ApiImplicitParam(name = "tableColumnList", value = "表字段数组json", required = true) })
	public R<String> uploadToDs(MultipartFile file, String tableName, String tableComment, Integer dsId,
			String tableColumnList) throws Exception {
		String extension = FilenameUtils.getExtension(file.getOriginalFilename());
		if (!Stream.of(CubeConstants.SUPPORTS_FILE).anyMatch(f -> f.equalsIgnoreCase(extension))) {
			return R.fail(String.format("文件只支持%s格式", String.join(",", CubeConstants.SUPPORTS_FILE)));
		}
		// 1.查询数据源
		ReportDatasource reportDatasource = reportDatasourceService.getById(dsId);
		if (reportDatasource == null) {
			return R.fail("数据源不存在");
		}
		DsTypeEnum dsTypeEnum = DsTypeEnum.resolve(reportDatasource.getDsType());

		// 表名需要特殊处理[ORACLE数据库,表名变大写]
		if (dsTypeEnum == DsTypeEnum.oracle || dsTypeEnum == DsTypeEnum.dameng) {
			tableName = tableName.toUpperCase();
		}
		// 保存文件
		String tempFileName = UUID.randomUUID().toString() + "." + extension;
		File cubeFile = new File(cubeDir + tempFileName);
		// 保存文件
		FileUtils.copyInputStreamToFile(file.getInputStream(), cubeFile);
		// 初始化文件对象
		CubeTableFile cubeTableFile = new CubeTableFile();
		cubeTableFile.setCreateTime(new Date());
		cubeTableFile.setCreateBy(SecureUtil.getNickName());
		cubeTableFile.setCreateUserId(SecureUtil.getUserId());
		cubeTableFile.setFileName(file.getOriginalFilename());
		cubeTableFile.setDatasourceId(dsId);
		cubeTableFile.setTableComment(tableComment);
		cubeTableFile.setTableName(tableName);
		cubeTableFile.setFileUri("uploads/cube/" + tempFileName);
		cubeTableFile.setFileSize(file.getSize());

		// 解析字段
		List<FileStructure> fileStructureList = JSON.parseArray(tableColumnList, FileStructure.class);
		// 主键放第一个
		fileStructureList.add(0, new FileStructure(CubeConstants.COLUMN_PRIMARY_KEY, ViewColumnTypeEnum.BIGINT, "主键ID", true));
		// 系统默认添加用户ID和主键
		fileStructureList.add(new FileStructure(CubeConstants.COLUMN_USER_ID, ViewColumnTypeEnum.BIGINT, "创建人ID", false));
		// 2.根据数据源构造建表语句
		List<String> sqlList = dialectContext.getDialect(dsTypeEnum).buildCreateTableSql(tableName, tableComment, fileStructureList);//DbCreateTableEnum.buildCreateTableSql(dsTypeEnum, tableName, tableComment, fileStructureList);
		log.info("SqlList: \n" + String.join("\n", sqlList));
		if (sqlList == null || sqlList.isEmpty()) {
			return R.fail("暂不支持的数据源");
		}
		// 3.创建表
		dynamicTableService.executeSql(reportDatasource, sqlList.stream().toArray(String[]::new));
		// 4.保存文件表信息
		cubeTableFileService.save(cubeTableFile);
		// 5.读取数据并批量插入
		CubeExcelAnalysisEventListener analysisEventListener = new CubeExcelAnalysisEventListener(dataList -> {});
		EasyExcel.read(file.getInputStream(), analysisEventListener).excelType(ExcelTypeEnum.valueOf(extension.toUpperCase())).sheet().doRead();
		dynamicTableService.batchSave(reportDatasource, tableName, SecureUtil.getUserId(), fileStructureList.stream().toArray(FileStructure[]::new), analysisEventListener.getDataList());
		return R.success("文件上传成功");
	}


	  @ApiOperation(value = "文件上传追加数据")
	  @PostMapping("/uploadAppend")
	  public R<String> uploadAppend(@RequestBody ExcelFormParam params) {
	  		//1.查询数据源
		  ReportDatasource reportDatasource = reportDatasourceService.getById(params.getDsId());
		  if (reportDatasource == null) {
		  	return R.fail("数据源不存在");
		  }
		  // 2.查询表结构
		  List<TableColumnDto> reportColumnList = dialectContext.getDialect(reportDatasource.getDsTypeSyl()).listTableColumnsByTableViewName(reportDatasource, params.getTableName());

		  //3.文件是否存在
		  String filePath = cubeDir.replace("/uploads/cube", "") + params.getFile();
		  log.info("filePath: " + filePath);
//		  File excelFile = new File("D:\\test.xlsx");
		  File excelFile = new File(filePath);
		  if (!excelFile.exists()) {
			  return R.fail("Excel文件不存在");
		  }
		  //是否清空数据
		  if (params.getIsTruncate()==1) {
			  String sql = "TRUNCATE TABLE " + params.getTableName();
			  dynamicTableService.executeSql(reportDatasource, sql);
		  }
		  Map<String, Integer> indexMap = params.getList().stream().collect(Collectors.toMap(ExcelFormParam.ExcelFieldIndex::getColumnName,ExcelFormParam.ExcelFieldIndex::getIndex));

		  List<FileStructure> fileStructureList = new ArrayList<FileStructure>();
		  for(TableColumnDto columnDto : reportColumnList) {
		  	if (indexMap.containsKey(columnDto.getColumnName())){
				fileStructureList.add(new FileStructure(
						columnDto.getColumnName(),
						CubeConstants.COLUMN_USER_ID.equalsIgnoreCase(columnDto.getColumnName()) ? ViewColumnTypeEnum.BIGINT : columnDto.getColumnType(),
						columnDto.getColumnComments(),indexMap.get(columnDto.getColumnName())));
			}
		  	if(columnDto.getPrimaryKeyStatus() == 1){
				if (columnDto.getDetailColumnType() != DetailColumnTypeEnum.INT){
					fileStructureList.add(new FileStructure(
							columnDto.getColumnName(),
							columnDto.getColumnType(),
							columnDto.getDetailColumnType(),
							columnDto.getColumnComments(),true));
				}
			}

		  }
		  try {
		  	CubeExcelAnalysisEventListener analysisEventListener = new CubeExcelAnalysisEventListener(dataList -> {});
		  	EasyExcel.read(excelFile, analysisEventListener).sheet().headRowNumber(params.getWithHeader() ? 1 : 0).doRead();
	                         dynamicTableService.batchSaveAppend(reportDatasource, params.getTableName(), SecureUtil.getUserId(), fileStructureList.stream().toArray(FileStructure[]::new),
	                         analysisEventListener.getDataList());
			 return R.success("追加数据成功");
		  } catch (Exception e) {
			 log.error("batchInsert Data Failed!!" + e.getMessage(), e);
			 return R.fail("追加数据失败");
		  }

	  }
}
