/**
 * 
 */
package org.springblade.cube.utils;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.Optional;
import java.util.regex.Matcher;
import java.util.regex.Pattern;
import java.util.stream.Collectors;
import java.util.stream.Stream;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.util.RandomUtil;
import lombok.extern.slf4j.Slf4j;
import net.sourceforge.pinyin4j.PinyinHelper;
import net.sourceforge.pinyin4j.format.HanyuPinyinCaseType;
import net.sourceforge.pinyin4j.format.HanyuPinyinOutputFormat;
import net.sourceforge.pinyin4j.format.HanyuPinyinToneType;
import net.sourceforge.pinyin4j.format.HanyuPinyinVCharType;
import net.sourceforge.pinyin4j.format.exception.BadHanyuPinyinOutputFormatCombination;
import org.springblade.core.tool.exception.ServiceException;
import org.springblade.cube.dto.VirtualColumnParam;
import org.springblade.cube.model.CubeTableColumn;
import org.springblade.cube.model.FileStructure;
import org.springblade.datasource.enums.ViewColumnTypeEnum;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.StrUtil;

/**
 * @author wookvn
 *
 */
@Slf4j
public class CubeUtils {
	
	final static Pattern CHINESE_PATTERN = Pattern.compile("[\u4e00-\u9fa5]");
	//(?:--)|
	final static Pattern SQL_INJECT_PATTERN = Pattern.compile("(/\\*(?:.|[\\n\\r])*?\\*/)|(\\b(update|delete|drop|insert|alter|set|truncate|declare|exec|master|execute)\\b)", Pattern.CASE_INSENSITIVE);
	//计算字段全局变量参数匹配
	final static Pattern VIRTUAL_COLUMN_PARAM_PATTERN = Pattern.compile("\\$\\{.*?\\}");
	
	/**
	 * 智能识别Excel列的类型
	 * 
	 * @param excelColumnIndex - 第几列
	 * @param dataList         - 数据集列表(一般Excel前10行)
	 * @return
	 */
	public static ViewColumnTypeEnum resolveDataType(Integer excelColumnIndex, List<Map<Integer, String>> dataList) {
		// 迭代
		int numberIndex = 0, datetimeIndex = 0, mobileIndex = 0;
		for (Map<Integer, String> dataMap : dataList) {
			String dataValue = dataMap.get(excelColumnIndex);
			if(StrUtil.isBlank(dataValue) || _isNumber(dataValue.trim())) {
				numberIndex++;
			}
			if(StrUtil.isBlank(dataValue) || _isDateTime(dataValue.trim())) {
				datetimeIndex++;
			}
			if(StrUtil.isBlank(dataValue) || _isMobile(dataValue.trim())) {
				mobileIndex++;
			}
		}
		if(mobileIndex == dataList.size()) {
			return ViewColumnTypeEnum.STRING;
		}else if(numberIndex == dataList.size()) {
			return ViewColumnTypeEnum.NUMBER;
		}else if(datetimeIndex == dataList.size()) {
			return ViewColumnTypeEnum.DATETIME;
		}
		return ViewColumnTypeEnum.STRING;
	}

	
	public static String buildInsertSql(String tableName, List<FileStructure> fileStructureList) {
		return buildInsertSql(tableName, fileStructureList.stream().toArray(FileStructure[]::new));
	}
	/**
	 * 创建插入SQL
	 * 
	 * @param tableName
	 * @param fileStructureArray
	 * @return
	 */
	public static String buildInsertSql(String tableName, FileStructure[] fileStructureArray) {
		// 字段
		StringBuilder preFieldSql = new StringBuilder("(");
		Stream.of(fileStructureArray).map(FileStructure::getColumnName).forEach(f -> preFieldSql.append(f).append(","));
		preFieldSql.deleteCharAt(preFieldSql.length() - 1).append(")");

		// 数值占位符
		StringBuilder preValueSql = new StringBuilder("(");
		Stream.of(fileStructureArray).forEach(f -> preValueSql.append("?,"));
		preValueSql.deleteCharAt(preValueSql.length() - 1).append(")");
		return String.format(CubeConstants.INSERT_INTO_SQL_FORMAT, tableName, preFieldSql.toString(),
				preValueSql.toString());
	}
	
	/**
	 * 格式化虚拟字段公式
	 * 
	 * @param columnFormula
	 * @param tableColumnMap
	 * @return
	 */
	public static VirtualColumnParam formatColumnFormula(String columnFormula, Map<String, CubeTableColumn> tableColumnMap) {
		VirtualColumnParam virtualColumnParam = new VirtualColumnParam();
		List<String> columnNames = new ArrayList<String>();
		Matcher matcher = CubeConstants.COLUMN_FORMULA_PATTERN.matcher(columnFormula);
		while (matcher.find()) {
			String group = matcher.group();
			String key = group.substring(1, group.length() - 1);
			CubeTableColumn cubeTableColumn = tableColumnMap.get(key);
			if (cubeTableColumn == null) {
				throw new ServiceException("未找到需要替换的key：" + key);
			}
			columnNames.add(cubeTableColumn.getColumnName());
			columnFormula = columnFormula.replace(group, cubeTableColumn.getTableAliasName() + "." + cubeTableColumn.getColumnName());
		}
		virtualColumnParam.setColumnNames(columnNames);
		virtualColumnParam.setColumnSql(columnFormula);
		return virtualColumnParam;
	}

	/**
	 * 计算字段片段SQL替换全局变量
	 * 
	 * @param virtualSqlFragment
	 * @param extraParams
	 * @return
	 */
	public static String formatVirtualSqlFragment(String virtualSqlFragment, Map<String, List<Object>> extraParams) {
		if (StrUtil.isEmpty(virtualSqlFragment)){
			return "";
		}
		Matcher matcher = VIRTUAL_COLUMN_PARAM_PATTERN.matcher(virtualSqlFragment);
		while (matcher.find()) {
			String group = matcher.group();
			String key = group.substring(2, group.length() - 1);
			//.filter(item -> StrUtil.isNotBlank(item.toString().trim()))
			List<String> paramValList = Optional.<List<Object>>ofNullable(extraParams.get(key)).orElse(new ArrayList<>()).stream().map(Object::toString).collect(Collectors.toList());
			if (CollectionUtil.isEmpty(paramValList)) {
				throw new ServiceException("计算字段替换全局变量失败|未找到需要替换的key：" + key);
			}
			virtualSqlFragment = virtualSqlFragment.replace(group, String.join(",", paramValList));
		}
		return virtualSqlFragment;
	}
	
	/**
	 * 是否包含中文字符串
	 * @param str
	 * @return
	 */
	public static boolean isContainsChinese(String str) {
		return CHINESE_PATTERN.matcher(str).find();
	}
	
	/**
	 * 校验是否Sql注入
	 * @param sql
	 * @return
	 */
	public static boolean checkSqlInject(String sql) {
		return SQL_INJECT_PATTERN.matcher(sql).find();
	}
	
	/**
	 * 是否数字 支持格式:[1-带空格的纯数值/2-带,千位符的纯数值/3-带%的百分数/4-货币]
	 * 
	 * @param value
	 * @return
	 */
	static boolean _isNumber(String value) {
		try {
			// 货币字符串替换掉¥/$/%,以及逗号
			value = value.replaceAll(CubeConstants.NUMBER_FORAMT_PATTERNS, "");
			Double doubleValue = new Double(value);
			//建表时，数字的最大字段长度30
			return doubleValue < Double.MAX_VALUE && value.length() <= 30;
		} catch (Exception e) {
		}
		return false;
	}
	
	/**
	 * 是否日期类型
	 * 
	 * @param value
	 * @return
	 */
	static boolean _isDateTime(String value) {
		try {
			DateUtil.parse(value, CubeConstants.DATE_FORMAT_PATTERNS);
			return true;
		} catch (Exception e) {
		}
		return false;
	}
	
	/**
	 * 是否手机号
	 * @param value
	 * @return
	 */
	static boolean _isMobile(String value) {
		return value.startsWith("1") && value.length() == 11;
	}

	/**
	 *
	 * @param value
	 * @return
	 */
	public static String getPinyin(String value, String separator){
		HanyuPinyinOutputFormat defaultFormat = new HanyuPinyinOutputFormat();
		// 输出设置，大小写，音标方式
		defaultFormat.setCaseType(HanyuPinyinCaseType.LOWERCASE);
		defaultFormat.setVCharType(HanyuPinyinVCharType.WITH_V);
		defaultFormat.setToneType(HanyuPinyinToneType.WITHOUT_TONE);
		try {
			value = StrUtil.removeAll(value, ' ', '#', '[', ']', '-', '_', '(', ')', '$', '%', '!', '=', ',', '.', '》', '《', '）', '（', '。', '㎡').replaceAll("m³", "");
			return PinyinHelper.toHanYuPinyinString(value.toLowerCase(), defaultFormat,
					separator,true);
		} catch (Exception e) {
			log.error(e.getMessage(), e);
		}
		return RandomUtil.randomString(RandomUtil.BASE_CHAR, 6);
	}

	/**
	 * 获取拼音首字母
	 * @param value
	 * @return
	 */
	public static String getPinyinFirstLetter(String value) {
		String separator = "";
		final String splitSeparator = StrUtil.isEmpty(separator) ? "#" : separator;
		final List<String> split = StrUtil.split(getPinyin(value, splitSeparator), splitSeparator);
		return CollUtil.join(split, separator, (s)->String.valueOf(s.length() > 0 ? s.charAt(0) : StrUtil.EMPTY));
	}
}