package org.springblade.visual.controller;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.github.xiaoymin.knife4j.annotations.ApiOperationSupport;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.springblade.core.boot.ctrl.BladeController;
import org.springblade.core.mp.support.Condition;
import org.springblade.core.tool.api.R;
import org.springblade.system.client.DataPermClient;
import org.springblade.visual.entity.VisualData;
import org.springblade.visual.entity.VisualSelfReportCategory;
import org.springblade.visual.service.VisualDataService;
import org.springblade.visual.service.VisualSelfReportCategoryService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.validation.Valid;
import java.util.Date;
import java.util.List;

/**
 * @Author: hepingguo
 * @Description:
 * @Date: 2023/3/20 16:38
 **/
@RestController
@RequestMapping("visual/self/report")
@Api(value = "自主报表分类", tags = "自主报表分类接口")
public class VisualSelfReportCategoryController extends BladeController {

    @Autowired
    private VisualSelfReportCategoryService visualSelfReportCategoryService;

    @Autowired
    private VisualDataService visualDataService;

    @Autowired
    private DataPermClient dataPermClient;

    /**
     * 详情
     */
    @GetMapping("/detail")
    @ApiOperationSupport(order = 1)
    @ApiOperation(value = "详情", notes = "传入visualCategory")
    public R<VisualSelfReportCategory> detail(VisualSelfReportCategory visualCategory) {
        VisualSelfReportCategory detail = visualSelfReportCategoryService.getOne(Condition.getQueryWrapper(visualCategory));
        return R.data(detail);
    }

    /**
     * 分页 可视化分类表
     */
    @GetMapping("/list")
    @ApiOperationSupport(order = 2)
    @ApiOperation(value = "分页", notes = "传入visualCategory")
    public R<List<VisualSelfReportCategory>> list(VisualSelfReportCategory visualCategory) {
        QueryWrapper<VisualSelfReportCategory> queryWrapper = Condition.getQueryWrapper(visualCategory);
        long currUserId = getUser().getUserId();
        List<VisualSelfReportCategory> dataList = visualSelfReportCategoryService.list(queryWrapper.lambda().orderByDesc(VisualSelfReportCategory::getSortNum));
        Boolean checkSuperPermRet = dataPermClient.checkSuperPermission(currUserId);
        if(checkSuperPermRet != null && checkSuperPermRet) { //是否有超级权限
            return R.data(dataList);
        }else {
            //查询所有报表有权限的分类ID集合
            List<Integer> classifyIdList = visualDataService.classifyIdListPerm(currUserId,1);
            //1.获取这些分类的父级ID[包含自己创建的分类父级别ID]
            dataList.stream().filter(f -> (classifyIdList.contains(f.getId()) || f.getCreateUserId().longValue() == currUserId) && f.getParentId() != 0).map(VisualSelfReportCategory::getParentId).forEach(classifyIdList::add);
            //2.删除不在权限内的分类[且createUserId不是本人的]
            dataList.removeIf(y -> !classifyIdList.contains(y.getId()) && y.getCreateUserId().longValue() != currUserId);
            return R.data(dataList);
        }
    }

    /**
     * 新增或修改 可视化分类表
     */
    @PostMapping("/submit")
    @ApiOperationSupport(order = 6)
    @ApiOperation(value = "新增或修改", notes = "传入visualCategory")
    public R<String> submit(@Valid @RequestBody VisualSelfReportCategory visualCategory) {
        if(visualCategory.getId() == null && visualCategory.getParentId() != null && visualCategory.getParentId() > 0) {
            //添加子级分类，判断父级分类是否有大屏模版
            long visualCount = visualDataService.count(Wrappers.<VisualData>lambdaQuery()
                    .eq(VisualData::getCategory, visualCategory.getParentId())
                    .eq(VisualData::getVisualDataType,1));
            if(visualCount > 0) {
                return R.fail("你选择的父级分类下有自主报表，不能新建子分类。");
            }
        }
        if(visualCategory.getId() == null) {
            visualCategory.setCreateUserId(getUser().getUserId());
            visualCategory.setCreateTime(new Date());
        }
        return R.status(visualSelfReportCategoryService.saveOrUpdate(visualCategory));
    }

    /**
     * 删除 可视化分类表
     */
    @PostMapping("/remove")
    @ApiOperationSupport(order = 8)
    @ApiOperation(value = "根据ID删除", notes = "传入id")
    public R<String> remove(@ApiParam(value = "主键", required = true) @RequestParam Integer id) {
        long count = visualSelfReportCategoryService.count(Wrappers.<VisualSelfReportCategory>lambdaQuery().eq(VisualSelfReportCategory::getParentId, id));
        if(count > 0) {
            return R.fail("该分类下有子分类，不能删除。");
        }
        long visualCount = visualDataService.count(Wrappers.<VisualData>lambdaQuery()
                .eq(VisualData::getCategory, id)
                .eq(VisualData::getVisualDataType,1));
        if(visualCount > 0) {
            return R.fail("该分类下有大屏模版，不能删除。");
        }
        return R.status(visualSelfReportCategoryService.removeById(id));
    }
}
